@@ -309,7 +309,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      * @param $id
      * @return string
      */
-    protected function getRelatedObjectName($property, $id)
+    public function getRelatedObjectName($property, $id)
     {
         return $this->getRelatedObject($property, $id)->getObjectName();
     }
@@ -535,7 +535,13 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
                 $this->connection
             );
         } catch (NotFoundError $e) {
-            throw new RuntimeException($e->getMessage(), 0, $e);
+            // Hint: eventually a NotFoundError would be better
+            throw new RuntimeException(sprintf(
+                'Unable to load object referenced from %s "%s", %s',
+                $this->getShortTableName(),
+                $this->getObjectName(),
+                lcfirst($e->getMessage())
+            ), $e->getCode(), $e);
         }
 
         $this->reallySet($name, $object->get('id'));
@@ -1241,6 +1247,9 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             throw new RuntimeException($e->getMessage(), 0, $e);
         }
 
+        // Eventually trigger loop detection
+        $this->listAncestorIds();
+
         foreach ($imports as $object) {
             $v = $object->getSingleResolvedProperty($key);
             if (null !== $v) {
@@ -1783,6 +1792,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
                 if ($zoneId = $this->getSingleResolvedProperty('zone_id')) {
                     return $this->getNameForZoneId($zoneId, $config);
                 }
+            } catch (NestingError $e) {
+                throw $e;
             } catch (Exception $e) {
                 return self::RESOLVE_ERROR;
             }
@@ -2354,12 +2365,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             );
         }
 
-        $str = sprintf(
-            "define %s {\n$name",
-            $type,
-            $name
-        );
-
+        $str = "define $type {\n$name";
         if ($this->isTemplate()) {
             $str .= c1::renderKeyValue('register', '0');
         }
@@ -2553,6 +2559,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             $type = 'templateChoiceHost';
         } elseif ($type === 'service_template_choice') {
             $type = 'TemplateChoiceService';
+        } elseif ($type === 'scheduled_downtime' || $type === 'scheduled-downtime') {
+            $type = 'ScheduledDowntime';
         }
 
         return 'Icinga\\Module\\Director\\Objects\\' . $prefix . ucfirst($type);
@@ -2724,9 +2732,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         }
 
         $plain = (array) $object->toPlainObject(false, false);
-        unset($plain['vars']);
-        unset($plain['groups']);
-        unset($plain['imports']);
+        unset($plain['vars'], $plain['groups'], $plain['imports']);
         foreach ($plain as $p => $v) {
             if ($v === null) {
                 // We want default values, but no null values
@@ -2768,6 +2774,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      * @param bool $skipDefaults
      * @param array|null $chosenProperties
      * @param bool $resolveIds
+     * @param bool $keepId
      * @return object
      * @throws NotFoundError
      */
@@ -2775,7 +2782,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         $resolved = false,
         $skipDefaults = false,
         array $chosenProperties = null,
-        $resolveIds = true
+        $resolveIds = true,
+        $keepId = false
     ) {
         $props = array();
 
@@ -2794,7 +2802,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         foreach ($p as $k => $v) {
             // Do not ship ids for IcingaObjects:
             if ($resolveIds) {
-                if ($k === 'id' && $this->hasProperty('object_name')) {
+                if ($k === 'id' && $keepId === false && $this->hasProperty('object_name')) {
                     continue;
                 }
 
@@ -3006,7 +3014,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     {
         $params = array();
 
-        if ($this->isApplyRule()) {
+        if ($this->isApplyRule() && ! $this instanceof IcingaScheduledDowntime) {
             $params['id'] = $this->get('id');
         } else {
             $params = array('name' => $this->getObjectName());
