@@ -3,23 +3,37 @@
 namespace Icinga\Module\Director\Web\Form;
 
 use Exception;
+use gipfl\IcingaWeb2\Url;
 use Icinga\Authentication\Auth;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Data\Db\DbObject;
 use Icinga\Module\Director\Data\Db\DbObjectWithSettings;
 use Icinga\Module\Director\Exception\NestingError;
+use Icinga\Module\Director\Hook\IcingaObjectFormHook;
 use Icinga\Module\Director\IcingaConfig\StateFilterSet;
 use Icinga\Module\Director\IcingaConfig\TypeFilterSet;
 use Icinga\Module\Director\Objects\IcingaTemplateChoice;
 use Icinga\Module\Director\Objects\IcingaCommand;
 use Icinga\Module\Director\Objects\IcingaObject;
 use Icinga\Module\Director\Util;
+use Icinga\Module\Director\Web\Form\Element\ExtensibleSet;
 use Icinga\Module\Director\Web\Form\Validate\NamePattern;
 use Zend_Form_Element as ZfElement;
 use Zend_Form_Element_Select as ZfSelect;
 
 abstract class DirectorObjectForm extends DirectorForm
 {
+    const GROUP_ORDER_OBJECT_DEFINITION = 20;
+    const GROUP_ORDER_RELATED_OBJECTS = 25;
+    const GROUP_ORDER_ASSIGN = 30;
+    const GROUP_ORDER_CHECK_EXECUTION = 40;
+    const GROUP_ORDER_CUSTOM_FIELDS = 50;
+    const GROUP_ORDER_CUSTOM_FIELD_CATEGORIES = 60;
+    const GROUP_ORDER_EVENT_FILTERS = 700;
+    const GROUP_ORDER_EXTRA_INFO = 750;
+    const GROUP_ORDER_CLUSTERING = 800;
+    const GROUP_ORDER_BUTTONS = 1000;
+
     /** @var IcingaObject */
     protected $object;
 
@@ -285,7 +299,7 @@ abstract class DirectorObjectForm extends DirectorForm
     }
 
     // TODO: move to a subform
-    protected function handleRanges(IcingaObject $object, & $values)
+    protected function handleRanges(IcingaObject $object, &$values)
     {
         if (! $object->supportsRanges()) {
             return;
@@ -318,7 +332,7 @@ abstract class DirectorObjectForm extends DirectorForm
         return $this->addElementsToGroup(
             $elements,
             'check_execution',
-            60,
+            self::GROUP_ORDER_CHECK_EXECUTION,
             $this->translate('Check execution')
         );
     }
@@ -368,7 +382,7 @@ abstract class DirectorObjectForm extends DirectorForm
         return $this->displayGroups[$group];
     }
 
-    protected function handleProperties(DbObject $object, & $values)
+    protected function handleProperties(DbObject $object, &$values)
     {
         if ($this->hasBeenSent()) {
             foreach ($values as $key => $value) {
@@ -422,6 +436,16 @@ abstract class DirectorObjectForm extends DirectorForm
                 $el = $this->getElement($k);
                 if ($el) {
                     $this->setInheritedValue($el, $inherited->$k, $origins->$k);
+                } elseif (substr($k, -3) === '_id') {
+                    $k = substr($k, 0, -3);
+                    $el = $this->getElement($k);
+                    if ($el) {
+                        $this->setInheritedValue(
+                            $el,
+                            $object->getRelatedObjectName($k, $v),
+                            $origins->{"${k}_id"}
+                        );
+                    }
                 }
             }
         }
@@ -559,7 +583,7 @@ abstract class DirectorObjectForm extends DirectorForm
                 array('HtmlTag', array('tag' => 'dl')),
                 'Fieldset',
             ),
-            'order' => 20,
+            'order' => self::GROUP_ORDER_OBJECT_DEFINITION,
             'legend' => $this->translate('Main properties')
         ));
 
@@ -605,12 +629,15 @@ abstract class DirectorObjectForm extends DirectorForm
             if (is_bool($inherited)) {
                 $inherited = $inherited ? 'y' : 'n';
             }
-            if (array_key_exists($inherited, $multi)) {
+            if (is_scalar($inherited) && array_key_exists($inherited, $multi)) {
                 $multi[null] = $multi[$inherited] . sprintf($txtInherited, $inheritedFrom);
             } else {
                 $multi[null] = $this->translate($this->translate('- inherited -'));
             }
             $el->setMultiOptions($multi);
+        } elseif ($el instanceof ExtensibleSet) {
+            $el->setAttrib('inherited', $inherited);
+            $el->setAttrib('inheritedFrom', $inheritedFrom);
         } else {
             if (is_string($inherited) || is_int($inherited)) {
                 $el->setAttrib('placeholder', $inherited . sprintf($txtInherited, $inheritedFrom));
@@ -659,9 +686,14 @@ abstract class DirectorObjectForm extends DirectorForm
         $object = $this->object();
 
         if ($object instanceof IcingaObject) {
+            $params = $object->getUrlParams();
+            $url = Url::fromPath($this->getAction());
+            if ($url->hasParam('dbResourceName')) {
+                $params['dbResourceName'] = $url->getParam('dbResourceName');
+            }
             $this->setSuccessUrl(
                 'director/' . strtolower($this->getObjectShortClassName()),
-                $object->getUrlParams()
+                $params
             );
         } elseif ($object->hasProperty('id')) {
             $this->setSuccessUrl($this->getSuccessUrl()->with('id', $object->getProperty('id')));
@@ -687,6 +719,24 @@ abstract class DirectorObjectForm extends DirectorForm
         return $this->object !== null;
     }
 
+    public function isIcingaObject()
+    {
+        if ($this->object !== null) {
+            return $this->object instanceof IcingaObject;
+        }
+
+        /** @var DbObject $class */
+        $class = $this->getObjectClassname();
+        $instance = $class::create();
+
+        return $instance instanceof IcingaObject;
+    }
+
+    public function isMultiObjectForm()
+    {
+        return false;
+    }
+
     public function setObject(DbObject $object)
     {
         $this->object = $object;
@@ -723,17 +773,17 @@ abstract class DirectorObjectForm extends DirectorForm
         return $this->objectName;
     }
 
-    protected function removeFromSet(& $set, $key)
+    protected function removeFromSet(&$set, $key)
     {
         unset($set[$key]);
     }
 
-    protected function moveUpInSet(& $set, $key)
+    protected function moveUpInSet(&$set, $key)
     {
         list($set[$key - 1], $set[$key]) = array($set[$key], $set[$key - 1]);
     }
 
-    protected function moveDownInSet(& $set, $key)
+    protected function moveDownInSet(&$set, $key)
     {
         list($set[$key + 1], $set[$key]) = array($set[$key], $set[$key + 1]);
     }
@@ -772,14 +822,18 @@ abstract class DirectorObjectForm extends DirectorForm
             $this->setDefaultsFromObject($this->object);
         }
         $this->prepareFields($this->object());
+        IcingaObjectFormHook::callOnSetup($this);
         if ($this->hasBeenSent()) {
             $this->handlePost();
         }
         try {
             $this->loadInheritedProperties();
             $this->addFields();
+            $this->callOnRequestCallables();
         } catch (Exception $e) {
             $this->addUniqueException($e);
+
+            return;
         }
 
         if ($this->shouldBeDeleted()) {
@@ -918,7 +972,7 @@ abstract class DirectorObjectForm extends DirectorForm
         return $this->getSentValue($name) === $this->getElement($name)->getLabel();
     }
 
-    protected function abortDeletion()
+    public function abortDeletion()
     {
         if ($this->hasDeleteButton()) {
             $this->setSentValue($this->deleteButtonName, 'ABORTED');
@@ -1154,7 +1208,7 @@ abstract class DirectorObjectForm extends DirectorForm
             return $this;
         }
 
-        if (in_array($this->getObjectShortClassName(), ['TimePeriod'])) {
+        if (in_array($this->getObjectShortClassName(), ['TimePeriod', 'ScheduledDowntime'])) {
             $required = false;
         } else {
             $required = $required !== null ? $required : !$this->isTemplate();
@@ -1508,7 +1562,7 @@ abstract class DirectorObjectForm extends DirectorForm
                 array('HtmlTag', array('tag' => 'dl')),
                 'Fieldset',
             ),
-            'order'  => 75,
+            'order'  => self::GROUP_ORDER_EXTRA_INFO,
             'legend' => $this->translate('Additional properties')
         ));
 
@@ -1541,7 +1595,7 @@ abstract class DirectorObjectForm extends DirectorForm
                 array('HtmlTag', array('tag' => 'dl')),
                 'Fieldset',
             ),
-            'order'  => 30,
+            'order'  => self::GROUP_ORDER_ASSIGN,
             'legend' => $this->translate('Assign where')
         ));
 
@@ -1610,7 +1664,7 @@ abstract class DirectorObjectForm extends DirectorForm
                 array('HtmlTag', array('tag' => 'dl')),
                 'Fieldset',
             ),
-            'order' =>70,
+            'order' => self::GROUP_ORDER_EVENT_FILTERS,
             'legend' => $this->translate('State and transition type filters')
         ));
 
