@@ -9,6 +9,27 @@
 
 SET sql_mode = 'STRICT_ALL_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,NO_ENGINE_SUBSTITUTION,PIPES_AS_CONCAT,ANSI_QUOTES,ERROR_FOR_DIVISION_BY_ZERO';
 
+CREATE TABLE director_daemon_info (
+  instance_uuid_hex VARCHAR(32) NOT NULL, -- random by daemon
+  schema_version SMALLINT UNSIGNED NOT NULL,
+  fqdn VARCHAR(255) NOT NULL,
+  username VARCHAR(64) NOT NULL,
+  pid INT UNSIGNED NOT NULL,
+  binary_path VARCHAR(128) NOT NULL,
+  binary_realpath VARCHAR(128) NOT NULL,
+  php_binary_path VARCHAR(128) NOT NULL,
+  php_binary_realpath VARCHAR(128) NOT NULL,
+  php_version VARCHAR(64) NOT NULL,
+  php_integer_size SMALLINT NOT NULL,
+  running_with_systemd ENUM('y', 'n') NOT NULL,
+  ts_started BIGINT(20) NOT NULL,
+  ts_stopped BIGINT(20) DEFAULT NULL,
+  ts_last_modification BIGINT(20) DEFAULT NULL,
+  ts_last_update BIGINT(20) DEFAULT NULL,
+  process_info MEDIUMTEXT NOT NULL,
+  PRIMARY KEY (instance_uuid_hex)
+) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE utf8mb4_bin;
+
 CREATE TABLE director_activity_log (
   id BIGINT(20) UNSIGNED AUTO_INCREMENT NOT NULL,
   object_type VARCHAR(64) NOT NULL,
@@ -17,7 +38,7 @@ CREATE TABLE director_activity_log (
   old_properties TEXT DEFAULT NULL COMMENT 'Property hash, JSON',
   new_properties TEXT DEFAULT NULL COMMENT 'Property hash, JSON',
   author VARCHAR(64) NOT NULL,
-  change_time DATETIME NOT NULL,
+  change_time TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
   checksum VARBINARY(20) NOT NULL,
   parent_checksum VARBINARY(20) DEFAULT NULL,
   PRIMARY KEY (id),
@@ -84,7 +105,7 @@ CREATE TABLE director_generated_config (
 
 CREATE TABLE director_generated_file (
   checksum VARBINARY(20) NOT NULL COMMENT 'SHA1(content)',
-  content MEDIUMTEXT NOT NULL,
+  content LONGTEXT NOT NULL,
   cnt_object INT(10) UNSIGNED NOT NULL DEFAULT 0,
   cnt_template INT(10) UNSIGNED NOT NULL DEFAULT 0,
   cnt_apply INT(10) UNSIGNED NOT NULL DEFAULT 0,
@@ -114,9 +135,9 @@ CREATE TABLE director_deployment_log (
   config_checksum VARBINARY(20) DEFAULT NULL,
   last_activity_checksum VARBINARY(20) NOT NULL,
   peer_identity VARCHAR(64) NOT NULL,
-  start_time DATETIME NOT NULL,
-  end_time DATETIME DEFAULT NULL,
-  abort_time DATETIME DEFAULT NULL,
+  start_time TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
+  end_time TIMESTAMP NULL DEFAULT NULL,
+  abort_time TIMESTAMP NULL DEFAULT NULL,
   duration_connection INT(10) UNSIGNED DEFAULT NULL
     COMMENT 'The time it took to connect to an Icinga node (ms)',
   duration_dump INT(10) UNSIGNED DEFAULT NULL
@@ -158,8 +179,17 @@ CREATE TABLE director_datalist_entry (
     ON UPDATE CASCADE
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
+CREATE TABLE director_datafield_category (
+  id INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
+  category_name VARCHAR(255) NOT NULL,
+  description TEXT DEFAULT NULL,
+  PRIMARY KEY (id),
+  UNIQUE KEY category_name (category_name)
+) ENGINE=InnoDB DEFAULT CHARSET=utf8;
+
 CREATE TABLE director_datafield (
   id INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
+  category_id INT(10) UNSIGNED DEFAULT NULL,
   varname VARCHAR(64) NOT NULL COLLATE utf8_bin,
   caption VARCHAR(255) NOT NULL,
   description TEXT DEFAULT NULL,
@@ -167,7 +197,12 @@ CREATE TABLE director_datafield (
 -- datatype_param? multiple ones?
   format enum ('string', 'json', 'expression'),
   PRIMARY KEY (id),
-  KEY search_idx (varname)
+  KEY search_idx (varname),
+  CONSTRAINT director_datafield_category
+    FOREIGN KEY category (category_id)
+    REFERENCES director_datafield_category (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
 CREATE TABLE director_datafield_setting (
@@ -184,7 +219,7 @@ CREATE TABLE director_datafield_setting (
 
 CREATE TABLE director_schema_migration (
   schema_version SMALLINT UNSIGNED NOT NULL,
-  migration_time DATETIME NOT NULL,
+  migration_time TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
   PRIMARY KEY(schema_version)
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
@@ -288,8 +323,8 @@ CREATE TABLE director_job (
   run_interval INT(10) UNSIGNED NOT NULL, -- seconds
   timeperiod_id INT(10) UNSIGNED DEFAULT NULL,
   last_attempt_succeeded ENUM('y', 'n') DEFAULT NULL,
-  ts_last_attempt DATETIME DEFAULT NULL,
-  ts_last_error DATETIME DEFAULT NULL,
+  ts_last_attempt TIMESTAMP NULL DEFAULT NULL,
+  ts_last_error TIMESTAMP NULL DEFAULT NULL,
   last_error_message TEXT DEFAULT NULL,
   PRIMARY KEY (id),
   UNIQUE KEY (job_name),
@@ -320,6 +355,7 @@ CREATE TABLE icinga_command (
   disabled ENUM('y', 'n') NOT NULL DEFAULT 'n',
   methods_execute VARCHAR(64) DEFAULT NULL,
   command TEXT DEFAULT NULL,
+  is_string ENUM('y', 'n') NULL,
   -- env text DEFAULT NULL,
   -- vars text DEFAULT NULL,
   timeout SMALLINT UNSIGNED DEFAULT NULL,
@@ -480,7 +516,7 @@ CREATE TABLE icinga_host (
   object_type ENUM('object', 'template') NOT NULL,
   disabled ENUM('y', 'n') NOT NULL DEFAULT 'n',
   display_name VARCHAR(255) DEFAULT NULL,
-  address VARCHAR(64) DEFAULT NULL,
+  address VARCHAR(255) DEFAULT NULL,
   address6 VARCHAR(45) DEFAULT NULL,
   check_command_id INT(10) UNSIGNED DEFAULT NULL,
   max_check_attempts MEDIUMINT UNSIGNED DEFAULT NULL,
@@ -1348,7 +1384,7 @@ CREATE TABLE import_source (
     'failing'
   ) NOT NULL DEFAULT 'unknown',
   last_error_message TEXT DEFAULT NULL,
-  last_attempt DATETIME DEFAULT NULL,
+  last_attempt TIMESTAMP NULL DEFAULT NULL,
   description TEXT DEFAULT NULL,
   PRIMARY KEY (id),
   UNIQUE INDEX source_name (source_name),
@@ -1405,8 +1441,8 @@ CREATE TABLE import_run (
   id INT(10) UNSIGNED AUTO_INCREMENT NOT NULL,
   source_id INT(10) UNSIGNED NOT NULL,
   rowset_checksum VARBINARY(20) DEFAULT NULL,
-  start_time DATETIME NOT NULL,
-  end_time DATETIME DEFAULT NULL,
+  start_time TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
+  end_time TIMESTAMP NULL DEFAULT NULL,
   succeeded ENUM('y', 'n') DEFAULT NULL,
   PRIMARY KEY (id),
   CONSTRAINT import_run_source
@@ -1483,9 +1519,12 @@ CREATE TABLE sync_rule (
     'endpoint',
     'zone',
     'timePeriod',
-    'serviceSet'
+    'serviceSet',
+    'scheduledDowntime',
+    'notification',
+    'dependency'
   ) NOT NULL,
-  update_policy ENUM('merge', 'override', 'ignore') NOT NULL,
+  update_policy ENUM('merge', 'override', 'ignore', 'update-only') NOT NULL,
   purge_existing ENUM('y', 'n') NOT NULL DEFAULT 'n',
   filter_expression TEXT DEFAULT NULL,
   sync_state ENUM(
@@ -1495,7 +1534,7 @@ CREATE TABLE sync_rule (
     'failing'
   ) NOT NULL DEFAULT 'unknown',
   last_error_message TEXT DEFAULT NULL,
-  last_attempt DATETIME DEFAULT NULL,
+  last_attempt TIMESTAMP NULL DEFAULT NULL,
   description TEXT DEFAULT NULL,
   PRIMARY KEY (id),
   UNIQUE INDEX rule_name (rule_name)
@@ -1527,7 +1566,7 @@ CREATE TABLE sync_run (
   id BIGINT(10) UNSIGNED AUTO_INCREMENT NOT NULL,
   rule_id INT(10) UNSIGNED DEFAULT NULL,
   rule_name VARCHAR(255) NOT NULL,
-  start_time DATETIME NOT NULL,
+  start_time TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
   duration_ms INT(10) UNSIGNED DEFAULT NULL,
   objects_deleted INT(10) UNSIGNED DEFAULT 0,
   objects_created INT(10) UNSIGNED DEFAULT 0,
@@ -1672,6 +1711,7 @@ CREATE TABLE icinga_dependency (
   disabled ENUM('y', 'n') NOT NULL DEFAULT 'n',
   apply_to ENUM('host', 'service') DEFAULT NULL,
   parent_host_id INT(10) UNSIGNED DEFAULT NULL,
+  parent_host_var VARCHAR(128) DEFAULT NULL,
   parent_service_id INT(10) UNSIGNED DEFAULT NULL,
   child_host_id INT(10) UNSIGNED DEFAULT NULL,
   child_service_id INT(10) UNSIGNED DEFAULT NULL,
@@ -1785,6 +1825,62 @@ CREATE TABLE icinga_timeperiod_exclude (
     ON UPDATE CASCADE
 );
 
+CREATE TABLE icinga_scheduled_downtime (
+  id INT(10) UNSIGNED AUTO_INCREMENT NOT NULL,
+  object_name VARCHAR(255) NOT NULL,
+  zone_id INT(10) UNSIGNED DEFAULT NULL,
+  object_type ENUM('object', 'template', 'apply') NOT NULL,
+  disabled ENUM('y', 'n') NOT NULL DEFAULT 'n',
+  apply_to ENUM('host', 'service') DEFAULT NULL,
+  assign_filter TEXT DEFAULT NULL,
+  author VARCHAR(255) DEFAULT NULL,
+  comment TEXT DEFAULT NULL,
+  fixed ENUM('y', 'n') DEFAULT NULL,
+  duration INT(10) UNSIGNED DEFAULT NULL,
+  with_services ENUM('y', 'n') NULL DEFAULT NULL,
+  PRIMARY KEY (id),
+  UNIQUE INDEX object_name (object_name),
+  CONSTRAINT icinga_scheduled_downtime_zone
+  FOREIGN KEY zone (zone_id)
+  REFERENCES icinga_zone (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE
+) ENGINE=InnoDB DEFAULT CHARSET=utf8;
+
+CREATE TABLE icinga_scheduled_downtime_inheritance (
+  scheduled_downtime_id INT(10) UNSIGNED NOT NULL,
+  parent_scheduled_downtime_id INT(10) UNSIGNED NOT NULL,
+  weight MEDIUMINT UNSIGNED DEFAULT NULL,
+  PRIMARY KEY (scheduled_downtime_id, parent_scheduled_downtime_id),
+  UNIQUE KEY unique_order (scheduled_downtime_id, weight),
+  CONSTRAINT icinga_scheduled_downtime_inheritance_downtime
+  FOREIGN KEY host (scheduled_downtime_id)
+  REFERENCES icinga_scheduled_downtime (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_scheduled_downtime_inheritance_parent_downtime
+  FOREIGN KEY host (parent_scheduled_downtime_id)
+  REFERENCES icinga_scheduled_downtime (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE
+) ENGINE=InnoDB DEFAULT CHARSET=utf8;
+
+CREATE TABLE icinga_scheduled_downtime_range (
+  scheduled_downtime_id INT(10) UNSIGNED AUTO_INCREMENT NOT NULL,
+  range_key VARCHAR(255) NOT NULL COMMENT 'monday, ...',
+  range_value VARCHAR(255) NOT NULL COMMENT '00:00-24:00, ...',
+  range_type ENUM('include', 'exclude') NOT NULL DEFAULT 'include'
+  COMMENT 'include -> ranges {}, exclude ranges_ignore {} - not yet',
+  merge_behaviour ENUM('set', 'add', 'substract') NOT NULL DEFAULT 'set'
+  COMMENT 'set -> = {}, add -> += {}, substract -> -= {}',
+  PRIMARY KEY (scheduled_downtime_id, range_type, range_key),
+  CONSTRAINT icinga_scheduled_downtime_range_downtime
+  FOREIGN KEY scheduled_downtime (scheduled_downtime_id)
+  REFERENCES icinga_scheduled_downtime (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+) ENGINE=InnoDB DEFAULT CHARSET=utf8;
+
 INSERT INTO director_schema_migration
   (schema_version, migration_time)
-  VALUES (157, NOW());
+  VALUES (171, NOW());
